@@ -34,11 +34,11 @@ Hypothesis::API - Wrapper for the hypothes.is web (HTTP) API.
 
 =head1 VERSION
 
-Version 0.11
+Version 0.12
 
 =cut
 
-our $VERSION = '0.11';
+our $VERSION = '0.12';
 
 =head1 SYNOPSIS
 
@@ -80,6 +80,9 @@ Search functionality (no login needed):
         push @annotations, $item;
     }
 
+    my $total = $H->search_total({limit => 100}, $page_size);
+    print "Reported $total total items.\n";
+
 =head1 EXPORT
 
 Currently nothing.
@@ -90,6 +93,12 @@ my $json = JSON->new->allow_nonref;
 $json->pretty(1);
 $json->canonical(1); 
 
+
+#
+# TODO: add getter/setter?
+#
+my $page_size_default = 20;
+
 has 'api_url' => (
     is        => 'ro',
     default   => 'https://hypothes.is/api',
@@ -282,7 +291,8 @@ sub delete_id {
                 return 0;
             }
         } else {
-            die "Received unexpected object: no 'deleted' entry present.";
+            warn "Received unexpected object: no 'deleted' entry present.";
+            return 0;
         }
     } else {
         die "Got $content_type; expected an ARRAY or HASH.";
@@ -418,22 +428,22 @@ sub search {
     if (not defined $query) {
         $query = {};
     }
-    if (not defined $query->{ 'limit' }) {
-        #Default at the time, but need to make explicit here:
-        $query->{ 'limit' } = 20;
-    }
-    if (not defined $page_size) {
-        #Default at the time, but need to make explicit here:
-        $page_size = 20;
-    }
     if ( defined $query->{ 'uri' } ) {
         $query->{ 'uri' } = $self->uri_encoder->encode(
            $query->{ 'uri' }
         );
     }
+    if (not defined $page_size) {
+        #Default at the time, but need to make explicit here:
+        $page_size = $page_size_default;
+    }
+    if ( not defined $query->{ 'limit' } ) {
+        #Default at the time, but need to make explicit here:
+        $query->{ 'limit' } = $page_size;
+    }
 
     my $done = 0;
-    my $next_buf_start;
+    my $last_id = undef;
     my $num_returned = 0;
     my $limit_orig = $query->{ 'limit' };
     $query->{ 'limit' } = $page_size + 1;
@@ -447,21 +457,34 @@ sub search {
             my $url = URI->new( "${\$self->api_url}/search" );
             $url->query_form($query);
             warn $url, "\n" if $VERB > 1;
-            my $response;
-            my $json_content;
-            $response = $self->ua->get( $url );
-            $json_content = $json->decode($response->content);
+            my $response = $self->ua->get( $url );
+            my $json_content = $json->decode($response->content);
             @annotation_buff = @{$json_content->{ 'rows' }};
-            if ($limit_orig eq 'Infinity') {
+            if (defined $limit_orig and $limit_orig eq 'Infinity') {
                 # OK, we get the point, but let's get finite.
                 $limit_orig = $json_content->{ 'total' };
                 $query->{ 'limit' } = $json_content->{ 'total' };
             }
-            if (defined $next_buf_start) {
+            if (not defined $limit_orig or $json_content->{ 'total' } < $limit_orig) {
+                # No limit set or more than total. Set it to the total
+                # so we don't have to try an extra request past the 
+                # total number of results
+                $limit_orig = $json_content->{ 'total' };
+                warn "setting limit_orig=$limit_orig based on total\n" if $VERB > 1;
+            }
+            if (defined $last_id) {
                 # This assumes that the feed is like a stack: LIFO.
                 # Annotations created after the search call
                 # shouldn't be returned.
-                while (@annotation_buff && $next_buf_start->{'id'} ne $annotation_buff[0]->{'id'}) {
+                # 
+                # This is not the first query because $last_id is set and the
+                # offset arranges so that, without the addition of new 
+                # annotations, the first result from the new query will be
+                # the same as the last result of the old query. If it isn't
+                # then we assume that new items have been added to the beginning
+                # and scan forward to find the id. The may be more than one
+                # page of scanning.
+                while (@annotation_buff and $last_id ne $annotation_buff[0]->{'id'}) {
                     warn "mismatch: scanning for last seen id\n" if $VERB > 0;
                     shift @annotation_buff;
                     if (@annotation_buff == 0) {
@@ -469,25 +492,78 @@ sub search {
                         goto QUERY;
                     }
                 }
+                if (@annotation_buff) {
+                    shift @annotation_buff;
+                }
             }
-            $next_buf_start = pop @annotation_buff;
-            $done = 1 if (@annotation_buff == 0);
             $query->{ 'offset' } += $page_size;
             warn $response->content if $VERB > 5;
-            # Handle edge case that look-ahead element is the last element:
-            if (($num_returned + 1) == $limit_orig 
-                 || $json_content->{ 'total' } == 1 ) {
-                $num_returned++;
-                return $next_buf_start;
-            }
         }
+        return undef if ($done or @annotation_buff == 0);
+        my $anno = shift @annotation_buff;
+        $last_id = $anno->{'id'};
         $num_returned++;
-        return undef if $done;
-        return shift @annotation_buff;
+        return $anno;
     }
 
 }
 
+=head2 search_total(\%query, $page_size)
+
+Specific interface to GET /api/search that simply 
+returns the total number of query results. See 
+the search subroutine for more details on parameters.
+
+=cut
+
+sub search_total {
+
+    # Note: try to keep the logic here the same as in the search
+    # function, or possibly remove code duplication.
+    #
+    # Start of code duplication:
+    #
+    my ($self, $query, $page_size) = @_;
+
+    my $h = HTTP::Headers->new;
+    $h->header(
+        'content-type' => 'application/json;charset=UTF-8', 
+        'x-csrf-token' => $self->csrf_token,
+    );
+    if (not defined $query) {
+        $query = {};
+    }
+    if ( defined $query->{ 'uri' } ) {
+        $query->{ 'uri' } = $self->uri_encoder->encode(
+           $query->{ 'uri' }
+        );
+    }
+    if (not defined $page_size) {
+        #Default at the time, but need to make explicit here:
+        $page_size = 20;
+    }
+    if ( not defined $query->{ 'limit' } ) {
+        #Default at the time, but need to make explicit here:
+        $query->{ 'limit' } = $page_size;
+    }
+
+    my $done = 0;
+    my $last_id = undef;
+    my $num_returned = 0;
+    my $limit_orig = $query->{ 'limit' };
+    $query->{ 'limit' } = $page_size + 1;
+    #
+    # End of code duplication:
+    #
+
+    my $url = URI->new( "${\$self->api_url}/search" );
+    $url->query_form($query);
+    warn $url, "\n" if $VERB > 1;
+    my $response = $self->ua->get( $url );
+    my $json_content = $json->decode($response->content);
+    return $json_content->{ 'total' };
+}
+
 
 =head2 update_id($id, \%payload)
 
